<?php
/**
 *  Copyright (C) 2010 - 2017  <Robbyn Gerhardt>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package       Webpackages
 * @author        Robbyn Gerhardt
 * @copyright     Copyright (c) 2010 - 2017, Robbyn Gerhardt (http://www.robbyn-gerhardt.de/)
 * @license       http://opensource.org/licenses/MIT	MIT License
 * @link          http://webpackages.de
 * @since         Version 2.0.0
 * @filesource
 */

namespace package\system\plugins\redis;

use package\implement\IPlugin;
use package\system\valueObjects\plugins\VOApplyPlugin;

class redis implements IPlugin
{
	/**
	 * @var \Redis
	 */
	private $redis;

	/**
	 * @var array
	 */
	private $ini = array();

	/**
	 * @var bool
	 */
	private $isConnected = false;

	public function getClassName()
	{
		return 'redis';
	}

	public function getApplyPlugin()
	{
		$apply                            = array();
		$plugin                           = new VOApplyPlugin();
		$plugin->call_position            = VOApplyPlugin::BEFORE;
		$plugin->all_dynamic_class        = true;
		$plugin->all_dynamic_method       = true;
		$plugin->replace_default_function = true;
		$plugin->call                     = array($this, 'template_display_redis_get');

		$apply[] = $plugin;

		$plugin2                     = new VOApplyPlugin();
		$plugin2->all_dynamic_class  = true;
		$plugin2->all_dynamic_method = true;
		$plugin2->call_position      = VOApplyPlugin::AFTER;
		$plugin2->call               = array($this, 'template_display_redis_set');

		$apply[] = $plugin2;

		return $apply;
	}

	/**
	 * Setzt das Template in den Redis
	 *
	 * @return void
	 */
	public function template_display_redis_set()
	{
		if($this->isConnected)
		{
			$args = func_get_args();

			$class   = $args[0];
			$method  = $args[1];
			$content = $args[2];

			if(!empty($content))
			{
				echo '<script>console.log("Save in redis: '.$class.':'.$method.'")</script>';
				$this->redis->setex($class.'_'.$method, $this->ini['redis_timeout'], $content);
			}
			else
			{
				echo '<script>console.log("Content is empty")</script>';
			}
		}
	}

	/**
	 * Kontrolliert ob zu diesem Template bereits etwas im Redis existiert
	 *
	 * @return string
	 */
	public function template_display_redis_get()
	{
		if($this->isConnected)
		{
			$args = func_get_args();

			$class  = $args[0];
			$method = $args[1];

			$get = $this->redis->get($class.'_'.$method);

			if($get !== false)
			{
				echo '<script>console.log("Load from redis")</script>';
				echo $get;

				return true;
			}
			else
			{
				echo '<script>console.log("not in redis")</script>';
			}
		}

		return null;
	}

	public function construct()
	{
		if(class_exists('\Redis'))
		{
			$this->ini = parse_ini_file(__DIR__.DIRECTORY_SEPARATOR.'config.ini');

			$this->redis = new \Redis();

			if(@$this->redis->connect($this->ini['redis_host'], $this->ini['redis_port']))
			{
				$this->isConnected = true;
				#$this->clear_cache();
			}
			else
			{
				echo '<script>console.log("Error: redis not connected")</script>';
			}
		}
	}

	public function clear_cache()
	{
		if($this->isConnected)
		{
			$keys = $this->redis->keys("*");
			foreach($keys as $key)
			{
				$this->redis->del($key);
			}
		}
	}

	public function setAllClasses($allClasses)
	{
		// TODO: Implement setAllClasses() method.
	}
}